// Copyright 2021 Chaos Mesh Authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package v1alpha1

import (
	"reflect"

	"github.com/pkg/errors"
	"k8s.io/apimachinery/pkg/util/validation/field"

	"github.com/chaos-mesh/chaos-mesh/api/genericwebhook"
)

// validateDeviceName validates the gcp chaos actions
func (in GCPChaosAction) Validate(root interface{}, path *field.Path) field.ErrorList {
	allErrs := field.ErrorList{}

	switch in {
	case NodeStop, DiskLoss:
	case NodeReset:
	default:
		err := errors.WithStack(errUnknownAction)
		log.Error(err, "Wrong GCPChaos Action type")

		allErrs = append(allErrs, field.Invalid(path, in, err.Error()))
	}
	return allErrs
}

type GCPDeviceNames []string

// validateDeviceName validates the DeviceName
func (in *GCPDeviceNames) Validate(root interface{}, path *field.Path) field.ErrorList {
	allErrs := field.ErrorList{}
	obj := root.(*GCPChaos)
	if obj.Spec.Action == DiskLoss {
		if *in == nil {
			err := errors.Errorf("at least one device name is required on %s action", obj.Spec.Action)
			allErrs = append(allErrs, field.Invalid(path, *in, err.Error()))
		}
	}
	return allErrs
}

func init() {
	genericwebhook.Register("GCPDeviceNames", reflect.PtrTo(reflect.TypeOf(GCPDeviceNames{})))
}
